@@ -8,10 +8,10 @@ except ImportError:
     import unittest
 import os
 import hashlib
-import shutil
 from sumatra import commands, launch, datastore
 
-originals = [] # use for storing originals of mocked objects
+originals = []  # use for storing originals of mocked objects
+
 
 class MockDataStore(object):
     def __init__(self, root):
@@ -19,7 +19,7 @@ class MockDataStore(object):
     def generate_keys(self, *paths):
         return [datastore.DataKey(path, datastore.IGNORE_DIGEST) for path in paths]
     def contains_path(self, path):
-        return os.path.exists(path)
+        return os.path.isfile(path)
 
 
 class MockExecutable(object):
@@ -50,6 +50,8 @@ class MockRecordStore(object):
         return []
     def sync_all(self, other):
         return []
+    def update(self, project, field, value):
+        self.updated = (field, value)
 
 
 class MockRepository(object):
@@ -75,7 +77,7 @@ class MockRecord(object):
 
 class MockWorkingCopy(object):
     repository = MockRepository("http://mock_repository")
-    
+
 
 class MockProject(object):
     name = None
@@ -83,8 +85,10 @@ class MockProject(object):
     default_repository = "some repository"
     default_main_file = "walk_silly.py"
     default_executable = MockExecutable(path="a.out")
+    default_launch_mode = launch.SerialLaunchMode
     on_changed = "sound the alarm"
     data_label = "pluck from the ether"
+    allow_command_line_parameters = True
     record_store = MockRecordStore("default")
     saved = False
     info_called = False
@@ -106,11 +110,11 @@ class MockProject(object):
         self.launch_args.update(parameters=parameters,
                                 input_data=input_data,
                                 script_args=script_args)
-    def format_records(self, tags, mode, format):
-        self.format_args = {"tags": tags, "mode": mode, "format": format}
+    def format_records(self, format='text', mode='short', tags=None, reverse=False):
+        self.format_args = {"tags": tags, "mode": mode, "format": format, "reverse": reverse}
     def delete_record(self, label, delete_data=False):
         if "nota" in label:
-            raise KeyError # or just emit a warning?
+            raise KeyError  # or just emit a warning?
         else:
             self._records_deleted.append(label)
     def delete_by_tag(self, tag, delete_data=False):
@@ -118,8 +122,14 @@ class MockProject(object):
     def export(self): self.exported = True
     def most_recent(self):
         return MockRecord("most_recent")
-    def add_comment(self, label, comment):
-        self.comments[label] = comment
+    def add_comment(self, label, comment, replace=False):
+        if replace:
+            self.comments[label] = comment
+        else:
+            if label in self.comments:
+                self.comments[label] = self.comments[label] + "\n" + comment
+            else:
+                self.comments[label] = comment
     def add_tag(self, label, tag):
         self.tags[label] = tag
     def remove_tag(self, label, tag):
@@ -128,13 +138,19 @@ class MockProject(object):
         return False
     def show_diff(self, label1, label2, **kwargs):
         return "diff"
+    def repeat(self, original_label, new_label=None):
+        return (new_label or "repeated", original_label)
+    def change_record_store(self, new_store):
+        self.record_store = new_store
 
 
 def no_project():
     raise Exception("There is no Sumatra project here")
 
-def mock_mkdir(path, mode=0777):
-    print "Pretending to create directory %s" % path
+
+def mock_mkdir(path, mode=511):  # octal 777 "-rwxrwxrwx"
+    print("Pretending to create directory %s" % path)
+
 
 def mock_build_parameters(filename):
     if filename != "this.is.not.a.parameter.file":
@@ -150,6 +166,7 @@ def store_original(module, name):
     global originals
     originals.append((module, name, getattr(module, name)))
 
+
 def setup():
     store_original(os, "mkdir")
     os.mkdir = mock_mkdir
@@ -162,6 +179,7 @@ def setup():
     commands.get_record_store = MockRecordStore
     commands.FileSystemDataStore = MockDataStore
 
+
 def teardown():
     for item in originals:
         setattr(*item)
@@ -352,6 +370,11 @@ class ConfigureCommandTests(unittest.TestCase):
         self.assert_(not hasattr(self.prj.data_store, "archive_store"))
         self.prj.data_store = MockDataStore("/path/to/root")
 
+    def test_change_store(self):
+        new_store_path = "http://smt.example.com/records/"
+        commands.configure(["--store", new_store_path])
+        self.assertEqual(self.prj.record_store.path, new_store_path)
+
 
 class InfoCommandTests(unittest.TestCase):
 
@@ -416,6 +439,16 @@ class TestParseArguments(unittest.TestCase):
         self.assertEqual(script_args, "this.is.not.a.parameter.file")
         os.remove("this.is.not.a.parameter.file")
 
+    def test_with_arg_that_is_directory(self):
+        test_dir = "__pycache__"  # a directory that is likely to exist already, easier than creating one since we have mocked out os.mkdir
+        if os.path.exists(test_dir):
+            parameter_sets, input_data, script_args = commands.parse_arguments([test_dir], self.input_datastore)
+            self.assertEqual(parameter_sets, [])
+            self.assertEqual(input_data, [])
+            self.assertEqual(script_args, test_dir)
+        else:
+            raise unittest.SkipTest("test directory doesn't exist")
+
     def test_with_cmdline_parameters(self):
         f = open("test.param", 'w')
         f.write("a = 2\nb = 3\n")
@@ -459,13 +492,12 @@ class RunCommandTests(unittest.TestCase):
                     'label': None,
                     'input_data': [],
                     'reason': '',
-                    'version': 'latest',
-                    'launch_mode': launch.SerialLaunchMode(),
+                    'version': 'current',
                     'script_args': ''}
         self.assertEqual(self.prj.launch_args, expected)
 
     def test_with_single_script_arg(self):
-        commands.run(["some_parameter_file"]) # file doesn't exist so is treated as argument
+        commands.run(["some_parameter_file"])  # file doesn't exist so is treated as argument
         self.assertEqual(self.prj.launch_args,
                          {'executable': 'default',
                          'parameters': {},
@@ -473,16 +505,15 @@ class RunCommandTests(unittest.TestCase):
                          'label': None,
                          'input_data': [],
                          'reason': '',
-                         'version': 'latest',
-                         'launch_mode': launch.SerialLaunchMode(),
+                         'version': 'current',
                          'script_args': 'some_parameter_file'})
 
     def test_with_single_input_file(self):
-        data_content = "0.0 242\n0.1 2345\n0.2 42451\n"
-        f = open("this.is.not.a.parameter.file", 'w')
+        data_content = b"0.0 242\n0.1 2345\n0.2 42451\n"
+        f = open("this.is.not.a.parameter.file", 'wb')
         f.write(data_content)
         f.close()
-        commands.run(["this.is.not.a.parameter.file"]) # file exists but is not a parameter file so is treated as input data
+        commands.run(["this.is.not.a.parameter.file"])  # file exists but is not a parameter file so is treated as input data
         self.assertEqual(self.prj.launch_args,
                          {'executable': 'default',
                          'parameters': {},
@@ -490,8 +521,7 @@ class RunCommandTests(unittest.TestCase):
                          'label': None,
                          'input_data': [datastore.DataKey('this.is.not.a.parameter.file', hashlib.sha1(data_content).hexdigest())],
                          'reason': '',
-                         'version': 'latest',
-                         'launch_mode': launch.SerialLaunchMode(),
+                         'version': 'current',
                          'script_args': 'this.is.not.a.parameter.file'})
         os.remove("this.is.not.a.parameter.file")
 
@@ -499,8 +529,8 @@ class RunCommandTests(unittest.TestCase):
         f = open("test.param", 'w')
         f.write("a = 2\nb = 3\n")
         f.close()
-        data_content = "23496857243968b24cbc4275dc82470a\n"
-        f = open("this.is.not.a.parameter.file", 'w')
+        data_content = b"23496857243968b24cbc4275dc82470a\n"
+        f = open("this.is.not.a.parameter.file", 'wb')
         f.write(data_content)
         f.close()
         commands.run(["-l", "vikings", "-v", "234", "--reason='test'",
@@ -512,10 +542,10 @@ class RunCommandTests(unittest.TestCase):
                          'parameters': {'this': 'mock', 'a': 17, 'umlue': 43},
                          'main_file': 'main.py',
                          'label': 'vikings',
-                         'input_data': [datastore.DataKey('this.is.not.a.parameter.file', hashlib.sha1(data_content).hexdigest())],
+                         'input_data': [datastore.DataKey('this.is.not.a.parameter.file',
+                                                          hashlib.sha1(data_content).hexdigest())],
                          'reason': 'test',
                          'version': '234',
-                         'launch_mode': launch.SerialLaunchMode(),
                          'script_args': "spam <parameters> eggs this.is.not.a.parameter.file beans"})
         os.remove("this.is.not.a.parameter.file")
         os.remove("test.param")
@@ -524,6 +554,21 @@ class RunCommandTests(unittest.TestCase):
         # ought really to have a more specific Exception and to catch it so as to give a helpful error message to user
         self.assertRaises(Exception, commands.run, ["a=17", "umlue=43"])
 
+    def test_with_command_line_params_disallowed(self):
+        self.prj.allow_command_line_parameters = False
+        commands.run(["a=17", "umlue=43", "beans"])
+        self.assertEqual(self.prj.launch_args["script_args"],
+                         "a=17 umlue=43 beans")
+        self.prj.allow_command_line_parameters = True
+
+    def test_with_stdin_stdout(self):
+        with open("data.in", "w") as fp:
+            fp.write("1 2 3\n")
+        commands.run(["--stdin=data.in", "--stdout=data.out"])
+        self.assertEqual(self.prj.launch_args["script_args"],
+                         "< data.in > data.out")
+        os.remove("data.in")
+
 
 class ListCommandTests(unittest.TestCase):
 
@@ -574,11 +619,11 @@ class CommentCommandTests(unittest.TestCase):
     def test_single_arg_interpreted_as_comment_on_last_record(self):
         commands.comment(["that was amazing"])
         self.assertEqual(self.prj.comments["most_recent"], "that was amazing")
-    
+
     def test_two_args_interpreted_as_label_and_comment(self):
         commands.comment(["some_label", "that was appalling"])
         self.assertEqual(self.prj.comments["some_label"], "that was appalling")
-    
+
     #def test_invalid_label(self):
     #    self.fail()
     #
@@ -601,7 +646,7 @@ class TestTagCommand(unittest.TestCase):
     def test_single_arg_interpreted_as_tag_on_last_record(self):
         commands.tag(["foo"])
         self.assertEqual(self.prj.tags["most_recent"], "foo")
-    
+
     def test_tag_multiple_records(self):
         commands.tag(["foo", "a", "b", "c"])
         self.assertEqual(self.prj.tags["a"], "foo")
@@ -625,7 +670,10 @@ class RepeatCommandTests(unittest.TestCase):
 
     def test_repeat_last(self):
         commands.repeat(['last'])
-        
+
+    def test_repeat_with_user_specified_label(self):
+        commands.repeat(['last', '-l', 'new_label'])
+
 
 class DiffCommandTests(unittest.TestCase):
 
@@ -696,11 +744,22 @@ class SyncCommandTests(unittest.TestCase):
 
     def test_with_single_path(self):
         commands.sync(["/path/to/store"])
-        
+
     def test_with_two_paths(self):
         commands.sync(["/path/to/store1", "/path/to/store2"])
 
 
+class MigrateCommandTests(unittest.TestCase):
+
+    def setUp(self):
+        self.prj = MockProject()
+        commands.load_project = lambda: self.prj
+
+    def test_change_output_datastore(self):
+        commands.migrate(["--datapath", "/new/data/path"])
+        self.assertEqual(self.prj.record_store.updated, ("datastore.root", "/new/data/path"))
+
+
 class ArgumentParsingTests(unittest.TestCase):
 
     def test_parse_command_line_parameter_arg_must_contain_equals(self):
@@ -718,7 +777,7 @@ class ArgumentParsingTests(unittest.TestCase):
 
     def test_parse_command_line_parameter_with_list(self):
         result = commands.parse_command_line_parameter("c=[1,2,3,4,5]")
-        self.assertEqual(result, {'c': range(1,6)})
+        self.assertEqual(result, {'c': [1, 2, 3, 4, 5]})
 
     def test_parse_command_line_parameter_with_tuple(self):
         result = commands.parse_command_line_parameter("d=('a','b','c')")
