@@ -44,7 +44,7 @@ class TestExecutable(unittest.TestCase):
     @unittest.skipUnless(check_output, "test requires Python 2.7")
     def test__init__with_only_prog_name__should_try_to_find_full_path(self):
         prog = Executable("ls")
-        actual_path = check_output("which ls", shell=True).strip()
+        actual_path = check_output("which ls", shell=True).decode('utf-8').strip()
         self.assertEqual(prog.path, actual_path)
         
     def test__init__should_find_version_if_possible(self):
