@@ -17,8 +17,26 @@ class MockDiffFormatter(object):
         return ""
 sumatra.projects.get_diff_formatter = lambda: MockDiffFormatter
 
+
+class MockRepository(object):
+    url = "http://svn.example.com"
+
+    def __eq__(self, other):
+        return self.url == other.url
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __getstate__(self):
+        return {}
+
+    def get_working_copy(self):
+        return MockWorkingCopy()
+
+
 class MockWorkingCopy(object):
     path = "/path/to/working/copy"
+    repository = MockRepository()
     def has_changed(self):
         return False
     def use_latest_version(self):
@@ -31,23 +49,12 @@ class MockWorkingCopy(object):
         return True
     def get_username(self):
         return "The Knights Who Say Ni"
-        
-class MockRepository(object):
-    url = "http://svn.example.com"
-    
-    def __eq__(self, other):
-        return self.url == other.url
 
-    def __getstate__(self):
-        return {}
-
-    def get_working_copy(self):
-        return MockWorkingCopy()
 
 
 class MockExecutable(object):
     name = "Python"
-    path = sys.executable #"/usr/local/bin/python"
+    path = sys.executable  # "/usr/local/bin/python"
     version = sys.version
     requires_script = True
     def write_parameters(self, params, filename):
@@ -65,23 +72,32 @@ class MockLaunchMode(object):
         return True
     def __getstate__(self):
         return {}
-    
-    
+
+
 class MockSet(object):
     def add(self, x):
         self.added = x
     def remove(self, x):
         self.removed = x
 
-    
+
 class MockRecord(object):
     def __init__(self, label):
         self.label = label
         self.tags = MockSet()
+        self.parameters = {}
+        self.repository = MockRepository()
+        self.input_data = []
+        self.script_arguments = "-q"
+        self.executable = MockExecutable()
+        self.main_file = "admin.php"
+        self.version = "42"
+        self.launch_mode = MockLaunchMode()
+        self.outcome = ""
     def difference(r1, r2, igm, igf):
         return ""
 
-    
+
 class MockRecordStore(object):
     def save(self, project_name, record):
         pass
@@ -101,7 +117,7 @@ class MockRecordStore(object):
 
 
 class TestProject(unittest.TestCase):
-    
+
     def tearDown(self):
         if os.path.exists(".smt"):
             shutil.rmtree(".smt")
@@ -109,15 +125,15 @@ class TestProject(unittest.TestCase):
             os.rmdir("Data")
         if os.path.exists("test.py"):
             os.remove("test.py")
-    
+
     def write_test_script(self, filename):
         with open(filename, "w") as f:
             f.write("a=2\n")
-    write_test_script.__test__ = False # stop nose treating this as a test
-    
+    write_test_script.__test__ = False  # stop nose treating this as a test
+
     def test__init__with_minimal_arguments(self):
         proj = Project("test_project", record_store=MockRecordStore())
-        
+
     def test__creating_a_second_project_in_the_same_dir_should_raise_an_exception(self):
         proj1 = Project("test_project1", record_store=MockRecordStore())
         self.assertRaises(Exception,Project, "test_project2")
@@ -125,7 +141,7 @@ class TestProject(unittest.TestCase):
     def test__info(self):
         proj = Project("test_project", record_store=MockRecordStore())
         proj.info()
-        
+
     def test_new_record_with_minimal_args_should_set_defaults(self):
         self.write_test_script("test.py")
         proj = Project("test_project",
@@ -177,7 +193,7 @@ class TestProject(unittest.TestCase):
                        record_store=MockRecordStore())
         proj.delete_record("foo")
         self.assertEqual(proj.record_store.deleted, "foo")
-        
+
     def test__delete_by_tag__calls_delete_by_tag_on_the_record_store(self):
         proj = Project("test_project",
                        record_store=MockRecordStore())
@@ -187,17 +203,17 @@ class TestProject(unittest.TestCase):
         proj = Project("test_project",
                        record_store=MockRecordStore())
         proj.add_comment("foo", "comment goes here")
-        
+
     def test__add_tag__should_call_add_on_the_tags_attibute_of_the_record(self):
         proj = Project("test_project",
                        record_store=MockRecordStore())
         proj.add_tag("foo", "new_tag")
-        
+
     def test__remove_tag__should_call_remove_on_the_tags_attibute_of_the_record(self):
         proj = Project("test_project",
                        record_store=MockRecordStore())
         proj.remove_tag("foo", "new_tag")
-        
+
     def test__show_diff(self):
         proj = Project("test_project",
                        record_store=MockRecordStore())
@@ -212,18 +228,41 @@ class TestProject(unittest.TestCase):
         proj.add_record(MockRecord("record2"))
         self.assertEqual(proj._most_recent, "record2")
         proj.delete_record("record2")
-        self.assertEqual(proj._most_recent, "last") # should really be "record1", but we are not testing RecordStore here
+        self.assertEqual(proj._most_recent, "last")  # should really be "record1", but we are not testing RecordStore here
 
+    def test__backup(self):
+        def fake_copytree(source, target):
+            pass
+        orig_copytree = shutil.copytree
+        shutil.copytree = fake_copytree
+        proj = Project("test_project",
+                       record_store=MockRecordStore())
+        backup_dir = proj.backup()
+        shutil.copytree = orig_copytree
+        assert "backup" in backup_dir
+
+    def test__repeat(self):
+        orig_gwc = sumatra.projects.get_working_copy
+        sumatra.projects.get_working_copy = MockWorkingCopy
+        orig_launch = Project.launch
+        Project.launch = lambda self, **kwargs: "new_record"
+        proj = Project("test_project",
+                       record_store=MockRecordStore())
+        proj.add_record(MockRecord("record1"))
+        proj.add_record(MockRecord("record2"))
+        self.assertEqual(proj.repeat("record1")[0], "new_record")
+        sumatra.projects.get_working_copy = orig_gwc
+        Project.launch = orig_launch
 
 
 class TestModuleFunctions(unittest.TestCase):
-    
+
     def tearDown(self):
         if os.path.exists(".smt"):
             shutil.rmtree(".smt")
         if os.path.exists("Data"):
             os.rmdir("Data")
-    
+
     def test__load_project__should_return_Project(self):
         proj1 = Project("test_project", record_store=MockRecordStore())
         assert os.path.exists(".smt/project")
