@@ -2,6 +2,7 @@
 Unit tests for the sumatra.recordstore package
 """
 
+from __future__ import unicode_literals
 try:
     import unittest2 as unittest
 except ImportError:
@@ -13,7 +14,8 @@ from django.core import management
 
 from sumatra.records import Record
 from sumatra.programs import register_executable, Executable
-from sumatra.recordstore import shelve_store, django_store, http_store, serialization
+from sumatra.recordstore import (shelve_store, django_store, http_store,
+                                 serialization, get_record_store)
 from sumatra.versioncontrol import vcs_list
 import sumatra.launch
 import sumatra.datastore
@@ -23,11 +25,14 @@ try:
 except ImportError:
     import simplejson as json
 import urlparse
+from sumatra.compatibility import string_type
+
 
 originals = []
 django_store1 = None
 django_store2 = None
 
+
 class MockExecutable(Executable):
     name = "a.out"
     path = "/usr/local/bin/a.out"
@@ -37,6 +42,7 @@ class MockExecutable(Executable):
         pass
 register_executable(MockExecutable, "a.out", "/usr/local/bin/a.out", [])
 
+
 class MockRepository(object):
     url = "http://svn.example.com/"
     upstream = None
@@ -44,20 +50,23 @@ class MockRepository(object):
     def __init__(self, *args, **kwargs):
         pass
 
+
 class MockLaunchMode(object):
     type = "SerialLaunchMode"
     def __getstate__(self):
         return {}
 
+
 class MockDataStore(object):
     type = "FileSystemDataStore"
     def __init__(self, **parameters):
-        pass
+        self.root = parameters.get("root", None) or "/tmp"
     def __getstate__(self):
-        return {'root': "/tmp"}
+        return {'root': self.root}
     def copy(self):
         return self
 
+
 class MockDependency(object):
     name = "some_module"
     path = "/usr/lib/python/some_module.py"
@@ -66,6 +75,7 @@ class MockDependency(object):
     module = "python"
     source = "http://git.example.com/"
 
+
 class MockPlatformInformation(object):
     architecture_bits = 32
     architecture_linkage = ""
@@ -77,6 +87,7 @@ class MockPlatformInformation(object):
     system_name = ""
     version = ""
 
+
 class MockParameterSet(object):
     def __init__(self, initialiser):
         pass
@@ -85,11 +96,12 @@ class MockParameterSet(object):
     def pop(self, k, d):
         return None
 
+
 class MockRecord(object):
 
     def __init__(self, label):
         self.label = label
-        self.timestamp = datetime.now() #datetime(1901, 6, 1, 12, 0, 0)
+        self.timestamp = datetime.now()  # datetime(1901, 6, 1, 12, 0, 0)
         self.reason = "because"
         self.duration = 7543.2
         self.outcome = None
@@ -110,6 +122,7 @@ class MockRecord(object):
         self.user = "michaelpalin"
         self.input_data = []
         self.script_arguments = "arg1 arg2"
+        self.repeats = None
 
     def __eq__(self, other):
         return self.label == other.label and self.duration == other.duration
@@ -118,12 +131,14 @@ class MockRecord(object):
 class MockProject(object):
     name = "TestProject"
 
+
 def clean_up():
     pass
     #for filename in ("test.db", "test2.db"):
     #    if os.path.exists(filename):
     #        os.remove(filename)
 
+
 def setup():
     global django_store1, django_store2
     clean_up()
@@ -137,6 +152,7 @@ def setup():
     django_store.db_config.configure()
     vcs_list.append(sys.modules[__name__])
 
+
 def teardown():
     global django_store1, django_store2
     del sumatra.launch.MockLaunchMode
@@ -218,7 +234,7 @@ class BaseTestRecordStore(object):
 
     def test_delete_nonexistent_label(self):
         self.add_some_records()
-        self.assertRaises(Exception, # could be KeyError or DoesNotExist
+        self.assertRaises(Exception,  # could be KeyError or DoesNotExist
                           self.store.delete,
                           self.project.name,
                           "notarecord")
@@ -226,7 +242,7 @@ class BaseTestRecordStore(object):
 
     def test_str(self):
         #this test is pointless, just to increase coverage
-        assert isinstance(str(self.store), basestring)
+        assert isinstance(str(self.store), string_type)
 
     def test_most_recent(self):
         self.add_some_records()
@@ -250,6 +266,12 @@ class BaseTestRecordStore(object):
         self.assertEqual(sorted(rec.label for rec in self.store.list(self.project.name)),
                          sorted(rec.label for rec in other_store.list(self.project.name)))
 
+    def test_update(self):
+        self.add_some_records()
+        self.store.update(self.project.name, "datastore.root", "/new/path/to/store")
+        updated_value, = set(rec.datastore.root for rec in self.store.list(self.project.name))
+        self.assertEqual(updated_value, "/new/path/to/store")
+
 
 class TestShelveRecordStore(unittest.TestCase, BaseTestRecordStore):
 
@@ -300,6 +322,7 @@ class MockResponse(object):
     def __init__(self, status):
         self.status = status
 
+
 def check_record(record):
     # this is a rather basic test. Should also check the keys of the
     # subsidiary dicts, and the types of the values
@@ -310,7 +333,7 @@ def check_record(record):
                                       "datastore", "outcome", "output_data",
                                       "dependencies", "input_data",
                                       "script_arguments", "stdout_stderr",
-                                      "input_datastore"])
+                                      "input_datastore", "repeats"])
 
 class MockCredentials(object):
         credentials = [['domain', 'username', 'password']]
@@ -327,8 +350,11 @@ class MockHttp(object):
         u = urlparse.urlparse(uri)
         parts = u.path.split("/")[1:-1]
         if self.debug:
-            print "\n<<<<<", uri, u.path, len(parts), method, body, headers, u.params, u.query
-        if len(parts) == 2: # record uri
+            print("\n<<<<< %s %s %d %s %s %s %s %s" % (uri, u.path, len(parts),
+                                                       method, body, headers,
+                                                       u.params, u.query))
+
+        if len(parts) == 2:  # record uri
             if method == "PUT":
                 record = json.loads(body)
                 check_record(record)
@@ -345,27 +371,32 @@ class MockHttp(object):
                 status = 200
             elif method == "DELETE":
                 self.records.pop(parts[1])
-                most_recent = u""
+                most_recent = ""
                 for record in self.records.itervalues():
                     if record["timestamp"] > most_recent:
                         most_recent = record["timestamp"]
                         self.last_record = record
                 content = ""
                 status = 204
-        elif len(parts) == 1: # project uri
-            if u.query:
-                tags = u.query.split("=")[1].split(",")
-                records = set([])
-                for tag in tags:
-                    records = records.union(["%s://%s/%s/%s/" % (u.scheme, u.netloc, parts[0],
-                                               path) for path in self.records.keys() if tag in self.records[path]['tags']])
-                records = list(records)
-            else:
-                records = ["%s://%s/%s/%s/" % (u.scheme, u.netloc, parts[0],
-                                                    path) for path in self.records.keys()]
-            content = json.dumps({"records": records})
-            status = 200
-        elif len(parts) == 3: # tagged records uri
+        elif len(parts) == 1:  # project uri
+            if method == "GET":
+                if u.query:
+                    tags = u.query.split("=")[1].split(",")
+                    records = set([])
+                    for tag in tags:
+                        records = records.union(["%s://%s/%s/%s/" % (
+                            u.scheme, u.netloc, parts[0], path)
+                            for path in self.records.keys() if tag in self.records[path]['tags']])
+                    records = list(records)
+                else:
+                    records = ["%s://%s/%s/%s/" % (u.scheme, u.netloc, parts[0], path)
+                               for path in self.records.keys()]
+                content = json.dumps({"records": records, "name": "TestProject", "description": ""})
+                status = 200
+            elif method == "PUT":
+                content = ""
+                status = 201
+        elif len(parts) == 3:  # tagged records uri
             if method == "DELETE":
                 tag = parts[2]
                 n = 0
@@ -375,11 +406,11 @@ class MockHttp(object):
                         n += 1
                 status = 200
                 content = str(n)
-        elif len(parts) == 0: # list projects uri
+        elif len(parts) == 0:  # list projects uri
             status = 200
             content = '[{"id": "TestProject"}]'
         if self.debug:
-            print ">>>>>", status, content
+            print(">>>>> %s %s" % (status, content))
         return MockResponse(status), content
 
 
@@ -410,7 +441,7 @@ class TestHttpRecordStore(unittest.TestCase, BaseTestRecordStore):
         s = pickle.dumps(self.store)
         del self.store
         unpickled = pickle.loads(s)
-        
+
     def test_process_url(self):
         url, username, password = http_store.process_url("http://foo:bar@example.com:8000/path/file.html")
         self.assertEqual(username, "foo")
@@ -420,10 +451,16 @@ class TestHttpRecordStore(unittest.TestCase, BaseTestRecordStore):
     def test_list_projects(self):
         self.assertEqual(self.store.list_projects(), [self.project.name])
 
+    def test_create_project(self):
+        self.store.create_project("NewProject", "A new test project", "A long description")
+
+    def test_project_info(self):
+        self.assertEqual(self.store.project_info("TestProject")["name"], "TestProject")
+
 
 class TestSerialization(unittest.TestCase):
     maxDiff = None
-    
+
     def test_build_record_v0p4(self):
         with open("example_0.4.json") as fp:
             record = serialization.build_record(json.load(fp))
@@ -434,8 +471,13 @@ class TestSerialization(unittest.TestCase):
             record = serialization.build_record(json.load(fp))
         self.assertEqual(record.label, "haggling")
 
-    def test_round_trip(self):
+    def test_build_record_v0p5(self):
         with open("example_0.5.json") as fp:
+            record = serialization.build_record(json.load(fp))
+        self.assertEqual(record.label, "haggling")    
+
+    def test_round_trip(self):
+        with open("example_0.6.json") as fp:
             data_in = json.load(fp)
         record = serialization.build_record(data_in)
         data_out = json.loads(serialization.encode_record(record, indent=2))
@@ -445,6 +487,32 @@ class TestSerialization(unittest.TestCase):
         serialization.encode_project_info("foo", "description of foo")
 
 
+class TestModuleFunctions(unittest.TestCase):
+
+    def tearDown(self):
+        for filename in ("test_record_store.shelf", "test_record_store.shelf.db"):
+            if os.path.exists(filename):
+                os.remove(filename)
+
+    def test_get_record_store_http(self, ):
+        self.assertIsInstance(get_record_store("http://records.example.com/"),
+                              http_store.HttpRecordStore)
+
+    def test_get_record_store_shelve(self):
+        store = shelve_store.ShelveRecordStore(shelf_name="test_record_store.shelf")
+        store.shelf[str("foo")] = "bar"
+        store.shelf.sync()
+        del store
+        assert os.path.exists("test_record_store.shelf") or os.path.exists("test_record_store.shelf.db")
+        self.assertIsInstance(get_record_store("test_record_store.shelf"),
+                              shelve_store.ShelveRecordStore)
+
+    def test_get_record_store_create_shelve(self):
+        assert not os.path.exists("test_record_store.shelf")
+        self.assertIsInstance(get_record_store("test_record_store.shelf"),
+                              shelve_store.ShelveRecordStore)
+    
+
 if __name__ == '__main__':
     setup()
     unittest.main()
