@@ -61,10 +61,7 @@ class BaseTestWorkingCopy(object):
         self.assertEqual(self.wc.diff(), "")
         self.change_file()
         assert "+omega = 42" in self.wc.diff()
-    
-    def test__current_version(self):
-        self.assertEqual(self.wc.current_version(), self.latest_version)
-    
+        
     def test__use_version(self):
         P = self.read_parameters("%s/default.param" % self.tmpdir)
         self.assertEqual(self.read_parameters("%s/default.param" % self.tmpdir)['seed'],
@@ -338,8 +335,9 @@ class TestSubversionRepository(unittest.TestCase, BaseTestRepository):
         r = self._create_repository()
         assert hasattr(r._client, "checkout")
         
-    def test__init__with_nonexistent_repos__should_raise_Exception(self):
-        self.assertRaises(Exception, SubversionRepository, "file:///tmp/chnseriguchs")
+    def test__exists__with_nonexistent_repos__should_raise_Exception(self):
+        r = SubversionRepository("file:///tmp/chnseriguchs")
+        self.assertFalse(r.exists)
    
     @unittest.skipIf(skip_ci, "Skipping test on CI server")
     def test__checkout__with_nonexistent_repos__should_raise_Exception(self):
